#!/usr/bin/python

# TempLux.py
# Import librerie
import pigpio
import math
import time
from array import array
from socket import *
from struct import *

# Definizione costanti
Host_server = "127.0.0.1" 
Port_server = 42001

adc_address1 = 0x48   # Imposta indirizzo 000 ADC 

adc_channel1 = 0x40   # 
adc_channel2 = 0x41   # Imposta indirizzi canali
adc_channel3 = 0x42   # con risoluzione di 8 bit
adc_channel4 = 0x43   # 

R1 = 15000.0          # Resistenza R1 
R2 = 15000.0          # Resistenza R2
R_th0 = 10000.0       # Resistenza riferimento termistore a 25 c
R_fr0 = 70000.0       # Resistenza fotoresistore con illuminamento IL unitario 

V_IN = 3.3            # Tensione alimentazione partitori tensione
V_REF = 3.3           # Tensione riferimento misura ADC
A = 0.00335402        # Steinhart-Hart Costante A
B = 0.000256985       # Steinhart-Hart Costante B
C = 2.62013e-6        # Steinhart-Hart Costante C
D = 6.38309e-8        # Steinhart-Hart Costante D

K = 6.0               # Fattore dissipazione K 6mV C

Pend = 0.7            # Valore Gamma fotoresistenza

# Inizio elaborazione
client = socket(AF_INET, SOCK_STREAM)
client.connect((Host_server, Port_server))

pi = pigpio.pi('localhost')

while True:
   data = client.recv(100)
   print data [4:30]
   if data[4:30] == 'broadcast "TempLux"':
      binlen = data [0:4]
      declen = unpack('BBBB', binlen)
      print declen

      time.sleep(0.1)

      # Lettura canale 0 ADC - un byte esadecimale
      # La prima istruzione imposta la richiesta di campionamento su ADC 0
      # Si eseguino tre letture per evitare 0x80 all'accensione e per la precisone 
      # incrementale del convertire

      h = pi.i2c_open(1, adc_address1) # open device at address 0x48 on bus 1
      print "handle = ",
      print h

      pi.i2c_write_byte(h, adc_channel1) # send byte 0x23 to device 2

      raw_val = pi.i2c_read_byte(h) # read a byte from device 2
      raw_val = pi.i2c_read_byte(h) # read a byte from device 2
      raw_val = pi.i2c_read_byte(h) # read a byte from device 2

      print "Hex ADC_0 = ",
      print hex(raw_val)
      # Elimina i primi due caratteri 0x dal valore letto ed eventuale L finale
      # Trasforma il risultato in decimale 
      hex_val = hex(raw_val)[2:].rstrip('L')
      dec_val = int(hex_val,16)
      print "Dec ADC_0 = ",
      print dec_val

      # Trasformazione in tensione del valore letto 
      V = (dec_val * V_REF) / 256.0

      print "Volt NTC = ",
      print V

      # Calcolo della resistenza 
      R_th = (R1 * V) / (V_IN - V)

      print "Resistenza NTC = ",
      print R_th

      # Calcolo dei gradi Kelvin con la formula di Steinhart-Hart 
      logR = math.log(R_th / R_th0)
      logR2 = logR**2
      logR3 = logR**3
      Stein = 1.0 / (A + B * logR + C * logR2 + D * logR3)

      # Conversione in gradi Celsius e applicazione fattore di dispersione
      Celsius = round(Stein - 273.15 - V**2 / (K * R_th),2)

      # Stampa del risultato
      print "Steinhart - Celsius = ",
      print Celsius

      # Lettura canale 1 ADC - un byte esadecimale
      # La prima istruzione imposta la richiesta di campionamento su ADC 1
      # Si eseguino tre letture per evitare 0x80 all'accensione e per la precisone 
      # incrementale del convertire

      pi.i2c_write_byte(h, adc_channel2) # send byte 0x23 to device 2

      raw_val2 = pi.i2c_read_byte(h) # read a byte from device 2
      raw_val2 = pi.i2c_read_byte(h) # read a byte from device 2
      raw_val2 = pi.i2c_read_byte(h) # read a byte from device 2

      print "Hex ADC_1 = ",
      print hex(raw_val2)

      pi.i2c_close(h)

      # Elimina i primi due caratteri 0x dal valore letto ed eventuale L finale
      # Trasforma il risultato in decimale 
      hex_val2 = hex(raw_val2)[2:].rstrip('L')
      dec_val2 = int(hex_val2,16)
      print "Dec ADC_1 = ",
      print dec_val2

      # Trasformazione in tensione del valore letto 
      V2 = (dec_val2 * V_REF) / 256.0

      print "Volt LUX = ",
      print V2

      # Calcolo della resistenza 
      R_lm = (R1 * V2) / (V_IN - V2)

      # Calcolo della luminosita con 2 decimali
      Lux = round((R_lm / R_fr0)**(1.0/-Pend),2)

      # Stampa del risultato
      print "Lux = ", 
      print Lux


      time.sleep(0.1)
     
      risp= 'sensor-update "temp" ' + str(Celsius) +' "lux" ' + str(Lux) 

      n = len(risp)
      a = array('c')
      a.append(chr((n >> 24) & 0xFF))
      a.append(chr((n >> 16) & 0xFF))
      a.append(chr((n >>  8) & 0xFF))
      a.append(chr(n & 0xFF))

      client.send(a.tostring() + risp)
      time.sleep(0.1)

      risp= 'broadcast "RispTempLux"'

      n = len(risp)
      a = array('c')
      a.append(chr((n >> 24) & 0xFF))
      a.append(chr((n >> 16) & 0xFF))
      a.append(chr((n >>  8) & 0xFF))
      a.append(chr(n & 0xFF))

      client.send(a.tostring() + risp)
      time.sleep(0.1)


pi.stop()











